/*===========================================================================
 * Project         : Mandriva Linux
 * Module          : rpmstats
 * File            : rpmstats.c
 * Version         : $Id: rpmstats.c 271355 2010-11-24 18:33:02Z peroyvind $
 * Author          : Frederic Lepied
 * Created On      : Sat Feb 17 16:08:53 2001
 *===========================================================================*/

static const char rcs_id[] = "$Id: rpmstats.c 271355 2010-11-24 18:33:02Z peroyvind $";
static const char compile_id[] = "$Compile: " __FILE__ " " __DATE__ " " __TIME__ " $";

/*
 **********************************************************************
 *
 * Included files
 *
 **********************************************************************
 */

#include "config.h"

#include <stdint.h>
#define _MIRE_INTERNAL
// We still just include this so that we can get rpmfiBuildFNames()
#include <rpm4compat.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/time.h>
#include <string.h>

int
main(int argc, char * argv[])
{
  rpmmi mi;
  rpmts ts;
  Header h;
  miRE mire;
  int32_t data_size;
  char ** file_list;
  struct timeval tv;
  int xx;

  gettimeofday(&tv, NULL);

#ifdef DEBUG
  printf("%d\n", tv.tv_sec);
#endif
  
  rpmReadConfigFiles(NULL, NULL);

  ts = rpmtsCreate();
  rpmtsSetRootDir(ts, "");
  if (rpmtsOpenDB(ts, O_RDONLY)) {
    rpmlogPrint(stderr);
    exit(EXIT_FAILURE);
  }

  mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES, NULL, 0);

  mire = mireNew(RPMMIRE_DEFAULT, 0);
  xx = mireSetCOptions(mire, RPMMIRE_REGEX, 0, REG_NOSUB | REG_EXTENDED | REG_NEWLINE, NULL);

  ts = rpmtsFree(ts);

  if (mireRegcomp(mire, "/usr/share/locale|/usr/share/icons|/usr/lib/menu|/etc/menu-methods") != 0) {
      rpmlogPrint(stderr);
      mire = mireFree(mire);
      exit(EXIT_FAILURE);
  }
  while ((h = rpmmiNext(mi)) != NULL) {
    int32_t type, count, loop;
    struct stat buf;
    int max = 0;
    char *file = NULL;
    char *nvr = NULL;
    HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));

    rpmdbCheckSignals();

    he->tag = RPMTAG_FILEFLAGS; 
    headerGet(h, he, 0);

    nvr = headerSprintf(h, "%{name}-%{version}-%{release}", NULL, NULL, NULL);

    if (nvr != NULL) {

      rpmfiBuildFNames(h, RPMTAG_BASENAMES, (const char ***)&file_list, &data_size);
      
      for(loop=0; loop<data_size; loop++) {
	if (mireRegexec(mire, file_list[loop], (size_t) 0) && !(he->p.ui32p[loop] & RPMFILE_DOC)) {
	  if (stat(file_list[loop], &buf) == 0) {
	    buf.st_atime = (buf.st_atime > buf.st_mtime) ? buf.st_atime : buf.st_mtime;
#ifdef DEBUG
	    fprintf(stderr, "%s %d\n", file_list[loop], buf.st_atime);
#endif
	    if (S_ISREG(buf.st_mode) && max < buf.st_atime) {
	      max = buf.st_atime;
	      file=file_list[loop];
	    }
	  } else if(!(he->p.ui32p[loop] & (RPMFILE_MISSINGOK | RPMFILE_GHOST ))) {
	      perror(file_list[loop]);
	  }
	} else {
#ifdef DEBUG
	  fprintf(stderr, "file %s match regex\n", file_list[loop]);
#endif
	}
      }
      if (max != 0) {
	printf("%ld %s %s\n", (tv.tv_sec-max)/(3600*24), nvr, file);
      } else {
	printf("0 %s\n", nvr);
      }
      free(file_list);
      free(nvr);
      free(he->p.ui32p);
    }
  }

  mire = mireFree(mire);
  mi = rpmmiFree(mi);

  return 0;
}
